/*	$OpenBSD: mplock.h,v 1.1 2018/08/06 18:39:13 kettenis Exp $	*/

/* public domain */

#ifndef _MACHINE_MPLOCK_H_
#define _MACHINE_MPLOCK_H_

#define __USE_MI_MPLOCK

#endif /* !_MACHINE_MPLOCK_H */
